# -*- coding: utf-8 -*-

'''
Data Analysis (:mod:`krakatoa.future.analysis`)
============================================================
'''

# ============================================================
# Imports
# ============================================================

import pandas as pd
import numpy as np
import seaborn as sns
from .preprocess import DataClean

from sklearn.decomposition import PCA

# ============================================================


class Analytics(DataClean):

    def __init__(self, target=None):
        super().__init__(target)
        pass

    def columnDist(self, column):

        # Verify if column is numeric or string
        # When numeric and unique values > min numer (eg: 50) histogram can be used
        # When string alwais use a count plots

        dtype = str(self.dataset[column].dtype)

        if dtype in ['object', 'category', 'string']:
            data_type = "count"
            values = self.dataset[column].value_counts()
            x = list(values.keys().astype("string"))
            y = list(values.values)

        else:
            # Check the amount of unique values
            percUnique = (
                self.dataset[column].nunique() / self.dataset.shape[0]) * 100
            countUnique = self.dataset[column].nunique()

            if percUnique > 10 or countUnique > 10:  # TODO revisitar esse percentual
                data_type = "histogram"
                # y = freq | x = edges
                y, x = np.histogram(self.dataset[column])
                '''
                Chart implementation ex:
                ax.bar(x[:-1], y, width=np.diff(x), edgecolor="black", align="edge")
                '''

            else:
                data_type = "count"
                values = self.dataset[column].value_counts()
                x = list(values.keys().astype("string"))
                y = list(values.values)

        result = {'type': data_type, 'label': list(x), 'values': list(y)}
        return result

    def targetDist(self, target):

        result = self.columnDist(target)

        self.distTarget = result

        return self.distTarget

    def checkTypes(self, threshold=90, changeDtypes=False):

        # Search for unique features and column types
        super().getColType()
        super()._getUniqueFeatures()

        textColumns = []
        catColumns = []

        # Check if any of the features represents more than threshold (default 90%)
        # If is text object, will help to identify if is category or text
        # when is above 90% generally is a text, otherwise is category
        unique = self.uniquePerc[self.uniquePerc >= threshold]
        if unique.shape[0] > 0:

            # Select category cols
            textColumns = list(
                unique[unique.keys().isin(self.category_cols)].keys())

        catColumns = [x for x in self.category_cols if x not in textColumns]

        self.textColumns = textColumns
        self.catColumns = catColumns

        # Converte o dtype das colunas de acordo com o que foi identificado
        if changeDtypes:
            conversion_dict = {
                "category": self.catColumns,
                "string": self.textColumns
            }

            for dtype, columns in conversion_dict.items():
                for col in columns:
                    self.dataset[col] = self.dataset[col].astype(dtype)

    def describe(self, dataset=None, load_from="dataframe", get_dist=False):

        if dataset != None:
            self.loadDataset(dataset=dataset, load_from=load_from)

        # Execute type checker function
        self.checkTypes(threshold=90, changeDtypes=True)

        # Describe with pandas
        pdDescribe = self.dataset.describe(percentiles=[], include="all")
        pdDescribe.fillna("", inplace=True)

        # Get columns types and add to describe dataframe
        columns_dtypes = list(self.dataset.dtypes.keys())
        values_dtype = list(self.dataset.dtypes.astype("string").values)
        dfDtypes = pd.DataFrame(
            data=[values_dtype], columns=columns_dtypes, index=['dtype'])

        # Add another unique counter
        columns_unique = list(self.uniqueCount.keys())
        values_unique = list(self.uniqueCount.astype("string").values)
        dfUnique = pd.DataFrame(
            data=[values_unique], columns=columns_unique, index=['nunique'])

        described = pd.concat([pdDescribe, dfDtypes, dfUnique])

        # Transform to dict
        self.described = described.to_dict()

        if get_dist:
            for col in described.keys():
                try:
                    self.described[col]['dist'] = self.columnDist(col)
                except:
                    self.described[col]['dist'] = {}

        return self.described

    def iqrOutliers(self, column: str, method: str = 'midpoint'):

        if column in self.numeric_cols:
            Q1 = np.percentile(self.dataset[column], 25, method=method)
            Q2 = np.percentile(self.dataset[column], 50, method=method)
            Q3 = np.percentile(self.dataset[column], 75, method=method)
            Q4 = np.percentile(self.dataset[column], 100, method=method)
            IQR = Q3 - Q1

            # Calculate bounds
            lower_bound = Q1-1.5*IQR
            rows_lower_bound = list(
                self.dataset[self.dataset[column] <= lower_bound].index)

            upper_bound = Q3+1.5*IQR
            rows_upper_bound = list(
                self.dataset[self.dataset[column] >= upper_bound].index)

            has_outliers = False if len(
                rows_lower_bound) + len(rows_upper_bound) == 0 else True
            return {
                'has_outliers': has_outliers,
                'lower_bound': lower_bound,
                'rows_lower_bound': rows_lower_bound,
                'upper_bound': upper_bound,
                'rows_upper_bound': rows_upper_bound,
                'Q1': Q1,
                'Q2': Q2,
                'Q3': Q3,
                'Q4': Q4,
                'IQR': IQR
            }

        else:
            raise ValueError(
                'Outliers calculations can be performed only on numeric columns!')

    def searchOutliers(self, method: str = 'midpoint'):
        outliers = {}
        for col in self.numeric_cols:

            iqr_outliers = self.iqrOutliers(col, method)

            if iqr_outliers['has_outliers']:
                outliers[col] = iqr_outliers

        return outliers

    def diagnostic(self):
        '''
        Disclaimer!

        This function returns some paremeters like -> 'missing', 'unique', 'target'

        When those returns as False, means that they are not ok and need attention!
        The True flag, means that there are no missing data, the target is solid or there are no unique columns!
        '''

        (rows, cols) = self.dataset.shape
        res_diagnostic = {'missing': True, 'unique': True,
                          'target': True, 'outliers': True, 'rows': rows, 'cols': cols}
        
        # Check for missing data
        null_data = self._nullPercFeatures()
        null_diagnostic = null_data[null_data['count'] > 0][[
            'variable', 'count']].to_dict('records')

        if len(null_diagnostic) > 0:
            null_diagnostic_items = {}
            for item in null_diagnostic:
                null_diagnostic_items[item['variable']] = item['count']
            res_diagnostic['missing'] = False
            res_diagnostic['missing_data'] = null_diagnostic_items

        # Check for unique data
        unique_data = self.dataset.nunique()
        unique_diagnostic = unique_data[unique_data == 1].to_dict()

        if len(unique_diagnostic.keys()) > 1:
            res_diagnostic['unique'] = False
            res_diagnostic['unique_data'] = unique_diagnostic

        # Check for valid target
        if self.target is None:
            res_diagnostic['target'] = False
        else:
            if self.target not in set(self.dataset.columns):
                res_diagnostic['target'] = False

        # Check for outliers
        for col in self.dataset.columns:
            try:
                outliers = self.iqrOutliers(column=col)
                if outliers['has_outliers']:
                    res_diagnostic['outliers'] = False
                    break
            except:
                pass

        return res_diagnostic

    def columnInfo(self, column: str, get_values: bool = True, get_values_method: str = 'head',
                   get_values_size: int = 10, most_common_size: int = 2):

        info = {
            'type': str(),
            'values': list(),
            'stats': dict(),
            'outliers': dict()
        }

        info['type'] = str(self.dataset[column].dtype)

        # Implement method to verify column types
        # Getting most common values
        values, count = np.unique(self.dataset[column], return_counts=True)
        count_sort_ind = np.argsort(-count)

        ordered_values = values[count_sort_ind]
        total_size = count.sum()
        perc_count = (count[count_sort_ind]/total_size)*100

        # top - most common values
        perc_sum = 0
        for k, (name, perc) in enumerate(zip(ordered_values, perc_count)):
            # check size
            if k >= most_common_size:
                # Calculate others
                info['stats']['most_common'].append(
                    {'name': 'other', 'perc': 100-round(perc_sum, 2)})
                break

            if 'most_common' not in info['stats'].keys():
                info['stats']['most_common'] = []

            info['stats']['most_common'].append(
                {'name': name, 'perc': round(perc, 2)})

            perc_sum += perc

        # unique values count
        info['stats'].update({'unique': self.dataset[column].nunique()})

        # unique values
        info['stats'].update(
            {'unique_values': list(self.dataset[column].unique())})

        # Missing values
        info['stats'].update(
            {'missing_values': int(self.dataset[column][self.dataset[column] == ''].count())})

        # Null values
        info['stats'].update(
            {'null_values': int(self.dataset[column][self.dataset[column].isnull()].count())})

        # Get outliers
        if column in self.numeric_cols:
            info['outliers'] = self.iqrOutliers(column)

            # get describe
            info['stats'].update(self.dataset[column].describe())

        # get column values
        if get_values:
            if get_values_method == 'tail':
                info['values'] = list(
                    self.dataset[column].tail(get_values_size).values)
            else:
                info['values'] = list(
                    self.dataset[column].head(get_values_size).values)

        return info

    def allColumnInfo(self, get_values: bool = True, get_values_method: str = 'head',
                      get_values_size: int = 10, ignore_columns: list = [],
                      most_common_size: int = 2, output_as: str = 'dict'):

        columns = [
            col for col in self.dataset.columns if col not in ignore_columns]

        if output_as == 'dict':
            result = {}
            for col in columns:
                result[col] = self.columnInfo(
                    col, get_values=get_values, get_values_method=get_values_method, get_values_size=get_values_size,
                    most_common_size=most_common_size)

            return result
        elif output_as == 'list':
            result = []
            for col in columns:
                dict_res = self.columnInfo(
                    col, get_values=get_values, get_values_method=get_values_method, get_values_size=get_values_size,
                    most_common_size=most_common_size)
                dict_res.update({'column': col})
                result.append(dict_res)
            return result
        else:
            raise ValueError(
                'The output must be one of the following : dict | list')

    def countPlot(self, column: str, plot: bool = False, **kwargs):

        try:
            # Output value counts
            df = self.dataset[self.dataset[column].notnull()][column]
            count = df.value_counts().to_dict()
            label, values = zip(*count.items())

            # Plot chart
            if plot:
                sns.countplot(x=df, **kwargs)

            return {
                'label': list(label),
                'values': list(values)
            }
        except:
            raise ValueError(
                'Could not count the data values. Check the data type and if the values are valids!')

    def histPlot(self, column: str, plot: bool = False, **kwargs):
        df = self.dataset[self.dataset[column].notnull()][column]

        y, x = np.histogram(df)

        if plot:
            sns.histplot(x=df, **kwargs)

        return {
            'label': list(x),
            'values': list(y)
        }

    def boxPlot(self, column: str, plot: bool = False, method: str = 'midpoint', **kwargs):
        df = self.dataset[self.dataset[column].notnull()][column]

        iqr = self.iqrOutliers(column, method=method)

        if plot:
            sns.boxplot(df, **kwargs)

        return iqr

    def pca(self, n_components=1, **kwargs):

        pca = PCA(n_components=n_components, **kwargs)

        pca.fit(self.dataset)

        return pca




# -*- coding: utf-8 -*-

'''
Quick Models(:mod:`krakatoa.models.quick`)
============================================================
'''

#============================================================
# Imports
#============================================================

from sklearn.model_selection import cross_validate

from ._getmodels import getModels
from ._metrics import getScores
import pandas as pd

#============================================================
# Regression Functions
#============================================================

class Regressor():
    def __init__(self):

        self._regression_scores = getScores('regression')

    def _crossvalidate(self, estimator, x, y, scoring, cv=5):

        result = cross_validate(estimator, x, y, scoring=scoring, cv=cv, n_jobs=-1)
        
        return result

    def _runModels(self, models, x, y, scoring, cv):
        results = {'estimator' : [], 'fit_time' : [], 'score_time' : []}
        
        for model in models:
            res = self._crossvalidate(model['estimator'], x, y, scoring, cv)
            
            results['estimator'].append(model['name'])

            for k, v in res.items():
                if k not in results.keys():
                    results[k] = []
                    
                results[k].append(v.mean())
        
        return results

    def linearRegression(self, x, y, score=['r2'], cv=5, **kwargs):
        '''
        Quick linear regression models evaluation.

        Parameters
        ----------
        x : DATAFRAME
            Dataset Features.
        y : NUMPY ARRAY
            Target.
        score : LIST, optional
            Scoring metrics. The default is ['r2'].
        cv : INT, optional
            Number of kfolds. The default is 5.

        Returns
        -------
        return : DATAFRAME
            Returns dataframe with models and selected metrics score.

        '''
        
        
        models = getModels(mode='regression', modelClasses=['linear'])
        
        scoring = []
        for s in score:
            scoring.append(self._regression_scores[s]['name'])
            # results['test_' + regression_scores[s]['name']]= []
            
        results = self._runModels(models, x, y, scoring, cv)
        
        return pd.DataFrame(results)

    def treeRegression(self, x, y, score=['r2'], cv=5, **kwargs):
        '''
        Quick linear regression models evaluation.

        Parameters
        ----------
        x : DATAFRAME
            Dataset Features.
        y : NUMPY ARRAY
            Target.
        score : LIST, optional
            Scoring metrics. The default is ['r2'].
        cv : INT, optional
            Number of kfolds. The default is 5.

        Returns
        -------
        return : DATAFRAME
            Returns dataframe with models and selected metrics score.

        '''
        
        models = getModels(mode='regression', modelClasses=['tree'])
        
        scoring = []
        for s in score:
            scoring.append(self._regression_scores[s]['name'])
            # results['test_' + regression_scores[s]['name']]= []
            
        results = self._runModels(models, x, y, scoring, cv)
        
        return pd.DataFrame(results)

    def boostRegression(self, x, y, score=['r2'], cv=5, **kwargs):
        '''
        Quick linear regression models evaluation.

        Parameters
        ----------
        x : DATAFRAME
            Dataset Features.
        y : NUMPY ARRAY
            Target.
        score : LIST, optional
            Scoring metrics. The default is ['r2'].
        cv : INT, optional
            Number of kfolds. The default is 5.

        Returns
        -------
        return : DATAFRAME
            Returns dataframe with models and selected metrics score.

        '''

        models = getModels(mode='regression', modelClasses=['boost'])
        
        scoring = []
        for s in score:
            scoring.append(self._regression_scores[s]['name'])
            
        results = self._runModels(models, x, y, scoring, cv)
        
        return pd.DataFrame(results)

    def multiRegression(self, x, y, models = ['boost', 'linear', 'tree'], score=['r2'], cv=5):
        
        models = getModels(mode='regression', modelClasses=models)
        
        scoring = []
        for s in score:
            scoring.append(self._regression_scores[s]['name'])
            
        results = self._runModels(models, x, y, scoring, cv)
        
        return pd.DataFrame(results)
    

#============================================================
# Classification Functions
#============================================================


class Classifier():

    def __init__(self):

        self._classification_scores = getScores('classification')

    def _crossvalidate(self, estimator, x, y, scoring, cv=5):

        result = cross_validate(estimator, x, y, scoring=scoring, cv=cv, n_jobs=-1)
        
        return result

    def _runModels(self, models, x, y, scoring, cv):
        results = {'estimator' : [], 'fit_time' : [], 'score_time' : []}
        
        for model in models:
            res = self._crossvalidate(model['estimator'], x, y, scoring, cv)
            
            results['estimator'].append(model['name'])

            for k, v in res.items():
                if k not in results.keys():
                    results[k] = []
                    
                results[k].append(v.mean())
        
        return results

    def svmClassifier(self, x, y, score=['accuracy'], cv=5, **kwargs):
        '''
        Quick SVM classification models evaluation.

        Parameters
        ----------
        x : DATAFRAME
            Dataset Features.
        y : NUMPY ARRAY
            Target.
        score : LIST, optional
            Scoring metrics. The default is ['accuracy'].
        cv : INT, optional
            Number of kfolds. The default is 5.

        Returns
        -------
        return : DATAFRAME
            Returns dataframe with models and selected metrics score.

        '''
        
        models = getModels(mode='classification', modelClasses=['svm'])
        
        scoring = []
        for s in score:
            scoring.append(self._classification_scores[s]['name'])
            
        results = self._runModels(models, x, y, scoring, cv)
        
        return pd.DataFrame(results)

    def treeClassifier(self, x, y, score=['accuracy'], cv=5, **kwargs):
        '''
        Quick Tree classification models evaluation.

        Parameters
        ----------
        x : DATAFRAME
            Dataset Features.
        y : NUMPY ARRAY
            Target.
        score : LIST, optional
            Scoring metrics. The default is ['accuracy'].
        cv : INT, optional
            Number of kfolds. The default is 5.

        Returns
        -------
        return : DATAFRAME
            Returns dataframe with models and selected metrics score.

        '''
        
        models = getModels(mode='classification', modelClasses=['tree'])
        
        scoring = []
        for s in score:
            scoring.append(self._classification_scores[s]['name'])
            
        results = self._runModels(models, x, y, scoring, cv)
        
        return pd.DataFrame(results)

    def boostClassifier(self, x, y, score=['accuracy'], cv=5, **kwargs):
        '''
        Quick Boost classification models evaluation.

        Parameters
        ----------
        x : DATAFRAME
            Dataset Features.
        y : NUMPY ARRAY
            Target.
        score : LIST, optional
            Scoring metrics. The default is ['accuracy'].
        cv : INT, optional
            Number of kfolds. The default is 5.

        Returns
        -------
        return : DATAFRAME
            Returns dataframe with models and selected metrics score.

        '''
        
        models = getModels(mode='classification', modelClasses=['boost'])
        
        scoring = []
        for s in score:
            scoring.append(self._classification_scores[s]['name'])
            
        results = self._runModels(models, x, y, scoring, cv)
        
        return pd.DataFrame(results)
    
  


# -*- coding: utf-8 -*-

'''
Configuration file(:mod:`krakatoa._getmodels`)
============================================================
'''

# ============================================================
# Imports
# ============================================================

from ._config import getConfigRegr, getConfigClass

# ============================================================
# Get models
# ============================================================
def getModels(mode, modelClasses, selMode = 'type', random_state=0):
    

    if mode == 'regression':
        config = getConfigRegr()
    elif mode == 'classification':
        config = getConfigClass()
    else:
        #TODO error implementation
        print('No configuration was selected')
        config = None


    if config != None:

        configVals = [x for x in config.values()] 
        result = []

        if selMode in ['type', 'name']:
            
            for m in modelClasses:
                res = list(filter(lambda x: x[selMode] == m, configVals))
                
                for r in res:
                    result.append(r)
        
        return result



# -*- coding: utf-8 -*-

'''
Data preprocessing (:mod:`krakatoa.future.preprocess`)
============================================================
'''

#============================================================
# Imports
#============================================================

import pandas as pd
from pandas.api.types import is_string_dtype, is_numeric_dtype, is_object_dtype, is_categorical_dtype, is_datetime64_dtype, is_float_dtype, is_integer_dtype
import numpy as np


from sklearn.model_selection import train_test_split
from sklearn.preprocessing import MinMaxScaler, StandardScaler, RobustScaler, MaxAbsScaler
from krakatoa.future.evaluate import countNull
from sklearn.preprocessing import OneHotEncoder, OrdinalEncoder, LabelEncoder

#============================================================
def changeColType(df, columns, newType):
    
    for col in columns:
        df[col] = df[col].astype(newType)
    

    return df

def splitDataset(x, y, test_size=0.3, random_state=0):
    x_train, x_test, y_train, y_test = train_test_split(x, 
                                                        y, 
                                                        test_size=test_size, 
                                                        random_state=random_state)
    
    y_train, y_test = np.array(y_train).ravel(), np.array(y_test).ravel()
    
    return x_train, x_test, y_train, y_test

def scale(dataset, columns, scaler='min_max', **kwargs):

    scalers = {
        'min_max' : MinMaxScaler(**kwargs),
        'max_abs' : MaxAbsScaler(**kwargs),
        'robust' : RobustScaler(**kwargs),
        'standard' : StandardScaler(**kwargs)
        }
    
    cur_scaler = scalers.get(scaler)
    dataset[columns] = cur_scaler.fit_transform(dataset[columns])
    
    return {
        "scaler" : cur_scaler,
        "dataset" : dataset
    }

def encode(dataset, column, encoder='one_hot_encoder', **kwargs):
    encoders = {
        'one_hot_encoder' : OneHotEncoder(**kwargs),
        'label_encoder' : LabelEncoder(),
        'ordinal_encoder' : OrdinalEncoder(**kwargs)
        }
    
    cur_encoder = encoders.get(encoder, None)

    if cur_encoder is None:
        raise ValueError(f'Encoder is not valid! Choose one of the following: {list(encoders.keys())}')

    if encoder == 'one_hot_encoder':
        transformed = cur_encoder.fit_transform(dataset[[column]])
        new_df = pd.DataFrame(transformed.toarray(), columns=cur_encoder.get_feature_names_out())
        dataset = dataset.join(new_df)
        dataset.drop(columns=[column], inplace=True)

    else:
        dataset[column] = cur_encoder.fit_transform(dataset[[column]])
    
    return {
        "encoder" : cur_encoder,
        "dataset" : dataset
    }

# Class designed to dataset management   
class DataClean():

    def __init__(self, target=None):
        self.dataset = pd.DataFrame()
        self.originalDataset = pd.DataFrame()
        self.target = target
    
    def _checkTarget(self):
        if self.target is None:
            raise ValueError('Target must be set in order to execute that function')

        return True

    def loadDataset(self, dataset, load_from="dataframe"):
        if load_from == "dataframe":
            self.dataset = dataset
            self.originalDataset = dataset
        elif load_from == "dict":
            self.dataset = pd.DataFrame(dataset)
            self.originalDataset = pd.DataFrame(dataset)
        else:
            print("Error! Select the right Dataset type and set it to load_from variable ('dataframe', 'dict')")
        self.getColType()
        self._getUniqueFeatures()
        self._nullPercFeatures()

    def getColType(self):
        
        # TODO precisamos ainda identificar colunas de outros tipos , como data e quando é numerica e categorica
        catCols = []
        numCols = []
        datCols = []
        floatCols = []
        intCols = []
        otherCols = []
        dtype = {}

        for k, v in self.dataset.dtypes.items():
            # Check for non numeric
            # if v in ['object', 'category']:
            if is_categorical_dtype(self.dataset[k]) or is_object_dtype(self.dataset[k]):
                # Try to set datetime data
                try:
                    # self.dataset[k] = pd.to_datetime(self.dataset[k], format='%d-%m-%Y %H:%M:%S.%f')
                    self.dataset[k] = pd.to_datetime(self.dataset[k], infer_datetime_format=True)
                    datCols.append(k)
                    dtype[k] = 'datetime'
                except:
                    catCols.append(k)
                    dtype[k] = 'string'

            # elif is_numeric_dtype(self.dataset[k]):
            #     numCols.append(k)
                # if v in ['float64', 'float32']:
            elif is_float_dtype(self.dataset[k]):
                floatCols.append(k)
                numCols.append(k)
                dtype[k] = 'float'
                # elif v in ['int64', 'int32', 'int16', 'int8', 'uint64', 'uint32', 'uint16', 'uint8']:
            elif is_integer_dtype(self.dataset[k]):
                intCols.append(k)
                numCols.append(k)
                dtype[k] = 'integer'
                    
            elif is_datetime64_dtype(self.dataset[k]):
                datCols.append(k)
                dtype[k] = 'datetime'
            else:
                otherCols.append(k)
                dtype[k] = 'other'
                
        self.category_cols = catCols
        self.numeric_cols = numCols
        self.integer_cols = intCols
        self.float_cols = floatCols
        self.other_cols = otherCols
        self.datetime_cols = datCols
        self.dtype = dtype
        if self.target is not None:
            self.target_col = [self.target]
        else:
            self.target_col = []

    def splitTrainTest(self):

        self._checkTarget()

        X = self.dataset.drop(columns=[self.target])
        y = self.dataset[self.target]

        self.X = X
        self.y = y

        return X, y

    def _nullPercFeatures(self):
        result = countNull(self.dataset)

        self.percNull = result
        return result

    def _getUniqueFeatures(self):
        dataset = self.dataset.copy()

        catUniqueCount = dataset[self.category_cols].nunique()
        catUniquePerc = (dataset[self.category_cols].nunique() / dataset.shape[0])*100
        uniqueCount = dataset.nunique()
        uniquePerc = (dataset.nunique() / dataset.shape[0])*100

        self.catUniqueCount = catUniqueCount
        self.catUniquePerc = catUniquePerc
        self.uniqueCount = uniqueCount
        self.uniquePerc = uniquePerc

    def dropColumns(self, columns):

        self.dataset.drop(columns=columns, inplace=True)
        self.getColType()
        return self.dataset
    
    def dropNaColumns(self, threshold=50):
        
        percNull = self._nullPercFeatures()

        self.dataset.drop(columns=list(percNull[percNull > threshold].keys()), inplace=True)
        self.getColType()

        return self.dataset
    
    def fillNa(self, strategy='mean'): #mean / most frequent | drop 
        # preencher com media | mais frequente (caso categorico)
        self.getColType()
        dataset = self.dataset.copy()
        if strategy == "mean":
            for numc in self.numeric_cols:
                mean = dataset[numc].mean()
                dataset[numc].fillna(mean, inplace=True)
            
            # categoricos por mais frequentes
            for catc in self.category_cols:
                mode = dataset[catc].mode()[0]
                dataset[catc].fillna(mode, inplace=True)
        elif strategy == "drop":
            dataset.dropna(inplace=True)

        self.dataset = dataset
        return self.dataset

    def cleanUnique(self, threshold=500):
        
        self._checkTarget()
        self._getUniqueFeatures()

        dataset = self.dataset.copy()
        col_drop = list(self.uniqueCount[self.uniqueCount > threshold].keys())

        # Dont drop target column for any reason | Dont drop numeric columns
        col_drop = [x for x in col_drop if x != self.target and x not in (self.numeric_cols)] 

        dataset.drop(columns=col_drop, inplace=True)

        self.dataset = dataset

        # refresh column data in self
        self.getColType()

        return self.dataset

    def encodeColumns(self, columns: list, encoder='one_hot_encoder'):
        
        for col in columns:

            res_encoder = encode(self.dataset, column=col, encoder=encoder)

        self.dataset = res_encoder["dataset"]
        self.encoder = res_encoder["encoder"]

        return self.dataset   

    def getDummies(self, columns = None):

        # self.dataset = pd.get_dummies(self.dataset)
        # return self.dataset
        self.dataset.reset_index(inplace=True, drop=True)

        # Instancia e fit one hot encoder
        hot = OneHotEncoder(handle_unknown='ignore', sparse=False)

        if columns is None:
            columns = self.category_cols        

        hot.fit(self.dataset[columns])
        hot_ds = hot.transform(self.dataset[columns])

        # Cria dataframe transformada
        hot_df = pd.DataFrame(hot_ds, columns = hot.get_feature_names_out(input_features=columns))

        # Concatena o dataframe do one hot com o original
        concat_df = pd.concat([self.dataset, hot_df], axis=1).drop(columns=columns, axis=1)
        self.oneHotEncoding = hot
        self.dataset = concat_df

        return self.dataset

    def scaleColumns(self, columns: list, scaler: str='min_max', **kwargs):

        res_scale = scale(self.dataset, columns=columns, scaler=scaler, **kwargs)

        self.dataset = res_scale["dataset"]
        self.scaler = res_scale["scaler"]

        return self.dataset

    def dataScale(self, scaler='min_max'):

        res_scale = scale(self.dataset, columns=self.numeric_cols, scaler=scaler)

        self.dataset = res_scale["dataset"]
        self.scaler = res_scale["scaler"]

        return self.dataset

    def pipeline(self, steps):

        _config = {
            'dataset' : self.loadDataset,
            'splitColType' : self.getColType,
            'dropNaCol' : self.dropNaColumns,
            'fillNa' : self.fillNa,
            'cleanUnique' : self.cleanUnique,
            'getDummies' : self.getDummies,
            'scale' : self.dataScale,
            'dropCol' : self.dropColumns,
            # 'target' : self.setTarget,
        }

        for step, args in steps:
            fun = _config.get(step) #Else mostra error TODO
            fun(**args)
            
        return self.dataset

    def fit_transform(self, dataset, dropThresh=50, fillNaStrategy='mean', uniqueThresh=500, scaler='min_max'): #Metodo automatizado | pipeline

        # self.loadDataset(dataset)
        # self.getColType()
        # self.dropNaColumns(threshold=dropThresh)
        # self.fillNa(strategy=fillNaStrategy)
        # self.cleanUnique(threshold=uniqueThresh)
        # self.getDummies()
        # self.scale(scaler=scaler)

        self.pipeline(steps=[
            ('dataset', {'dataset' : dataset}),
            ('splitColType', {}),
            ('dropNaCol', {'threshold' : dropThresh}),
            ('fillNa', {'strategy' : fillNaStrategy}),
            ('cleanUnique', {'threshold' : uniqueThresh}),
            ('getDummies', {}),
            ('scale', {'scaler' : scaler})
            ])
        
        return self.dataset

    def setColumnType(self, col:str, col_type:str):

        try:
            self.dataset[col] = self.dataset[col].astype(col_type)
            self.getColType()
            return True
        except Exception as e:
            print(e)
            return False
       
    def setTarget(self, target):
        self.target = target

    def consistFormat(self, target='mongodb'):

        if target == 'mongodb':

            # Check for NaT values in 
            for col in self.datetime_cols:
                if self.dataset[col].isnull().sum() > 0:
                    self.dataset[col] = self.dataset[col].astype('object').where(self.dataset[col].notnull(), None)

            # Transform dataframe to list and records
            records = self.dataset.to_dict('records')

            return records
        
        else:
            raise ValueError('Not implemented!')



from setuptools import setup

setup(
  name = 'krakatoa',       
  packages = ['krakatoa', 'krakatoa/models', 'krakatoa/future'],  
  version = '0.0.7',      
  license='MIT',        
  description = 'Machine Learning high level package.',  
  long_description='Machine Learning high level package.',  
  author = 'Matheus de Prá Andrade',              
  author_email = 'mpandrade@ucs.br',    
  url = 'https://github.com/aitec-mp/krakatoa',  
  download_url = 'https://github.com/aitec-mp/krakatoa/archive/refs/tags/0.0.7.tar.gz',    
  keywords = ['krakatoa', 'machine learning'],  
  install_requires=[    
          'scikit-learn',
          'numpy',
          'pandas',
          'xgboost',
          'seaborn'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',     
    'Intended Audience :: Developers',  
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11',
  ],
)
